/**
 * PrintInfo.cpp
 *
 * This file is part of the LibTiePie programming examples.
 *
 * Find more information on http://www.tiepie.com/LibTiePie .
 */

#include "PrintInfo.hpp"
#include <array>
#include <iomanip>
#include <iostream>
#include <sstream>

const std::array<std::string, TIEPIE_HW_GMN_COUNT> GeneratorModes{"Continuous", "Burst count", "Gated periods", "Gated", "Gated period start", "Gated period finish", "Gated run", "Gated run output", "Burst sample count", "Burst sample count output", "Burst segment count", "Burst segment count output"};
const std::array<std::string, TIEPIE_HW_CON_COUNT> ClockOutputTypes{"Disabled", "Sample", "Fixed"};
const std::array<std::string, TIEPIE_HW_CSN_COUNT> ClockSources{"External", "Internal"};
const std::array<std::string, TIEPIE_HW_CONNECTORTYPE_COUNT> ConnectorTypes{"BNC", "Banana", "Power plug"};
const std::array<std::string, TIEPIE_HW_CKN_COUNT> Couplings{"DCV", "ACV", "DCA", "ACA", "Ohm"};
const std::array<std::string, TIEPIE_HW_DEVICETYPE_COUNT> DeviceTypes{"Oscilloscope", "Generator"};
const std::array<std::string, TIEPIE_HW_FMN_COUNT> FrequencyModes{"Signal frequency", "Sample frequency"};
const std::array<std::string, TIEPIE_HW_MMN_COUNT> MeasureModes{"Stream", "Block"};
const std::array<std::string, TIEPIE_HW_ARMN_COUNT> AutoResolutionModes{"Disabled", "Native only", "All"};
const std::array<std::string, TIEPIE_HW_STN_COUNT> SignalTypes{"Sine", "Triangle", "Square", "DC", "Noise", "Arbitrary", "Pulse"};
const std::array<std::string, TIEPIE_HW_TCN_COUNT> TriggerConditions{"None", "Smaller", "Larger", "Inside", "Outside"};
const std::array<std::string, TIEPIE_HW_TKN_COUNT> TriggerKinds{"Rising edge", "Falling edge", "In window", "Out Window", "Any edge", "Enter window", "Exit window", "Pulse width positive", "Pulse width negative"};
const std::array<std::string, TIEPIE_HW_TLMN_COUNT> TriggerLevelModes{"Relative", "Absolute"};
const std::array<std::string, TIEPIE_HW_TOEN_COUNT> TriggerOutputEvents{"Generator start", "Generator stop", "Generator new period", "Oscilloscope running", "Oscilloscope triggered", "Manual"};
const std::array<std::string, 6> ServerErrorCodes = {"None", "Unknown", "Connection refused", "Network unreachable", "Timed out", "Hostname lookup failed"};
const std::array<std::string, 4> ServerStatuses = {"Disconnected", "Connecting", "Connected", "Disconnecting"};

void printLibraryInfo()
{
  try
  {
    std::cout << "Library:\n";
    std::cout << "  Version: " << versionToStr(*TiePie::Hardware::Library::version()) << '\n';
    std::cout << "  Configuration: " << std::hex << std::uppercase << "0x";
    const std::vector<uint8_t> Configuration(TiePie::Hardware::Library::config());
    for(std::vector<uint8_t>::size_type i = 0; i < Configuration.size(); i++)
      std::cout << std::setw(2) << std::setfill('0') << static_cast<unsigned int>(Configuration[i]);
    std::cout << std::dec << '\n';
  }
  catch(const std::exception& e)
  {
    std::cerr << "Exception: " << e.what() << '\n';
  }
}

void printDeviceInfo(const TiePie::Hardware::Device& dev)
{
  try
  {
    std::cout << "Device:\n";
    std::cout << "  Name                      : " << dev.name() << '\n';
    std::cout << "  Short name                : " << dev.nameShort() << '\n';
    std::cout << "  Serial number             : " << dev.serialNumber() << '\n';

    try
    {
      const std::string calibrationDate = dateToStr(dev.calibrationDate());
      std::cout << "  Calibration date          : " << calibrationDate << '\n';
    }
    catch(const std::exception&)
    {
    }

    std::cout << "  Product id                : " << dev.productId() << '\n';

    try
    {
      std::cout << "  IP address              : " << dev.ipAddress() << '\n';
    }
    catch(const std::exception&)
    {
    }

    try
    {
      const uint16_t ipPort = dev.ipPort();
      std::cout << "  IP port                   : " << ipPort << '\n';
    }
    catch(const std::exception&)
    {
    }

    std::cout << "  Has battery               : " << boolToStr(dev.hasBattery()) << '\n';
    if(dev.hasBattery())
    {
      std::cout << "  Battery:\n";
      try
      {
        const int batteryCharge = dev.batteryCharge();
        std::cout << "    Charge                  : " << batteryCharge << " %\n";
      }
      catch(const std::exception&)
      {
      }

      try
      {
        const int32_t batteryTimeToEmpty = dev.batteryTimeToEmpty();
        std::cout << "    Time to empty           : " << batteryTimeToEmpty << " minutes\n";
      }
      catch(const std::exception&)
      {
      }

      try
      {
        const int32_t batteryTimeToFull = dev.batteryTimeToFull();
        std::cout << "    Time to full            : " << batteryTimeToFull << " minutes\n";
      }
      catch(const std::exception&)
      {
      }

      try
      {
        const bool isBatteryChargerConnected = dev.isBatteryChargerConnected();
        std::cout << "    Charger connected       : " << boolToStr(isBatteryChargerConnected) << '\n';
      }
      catch(const std::exception&)
      {
      }

      try
      {
        const bool isBatteryCharging = dev.isBatteryCharging();
        std::cout << "    Charging                : " << boolToStr(isBatteryCharging) << '\n';
      }
      catch(const std::exception&)
      {
      }

      try
      {
        const bool isBatteryBroken = dev.isBatteryBroken();
        std::cout << "    Broken                  : " << boolToStr(isBatteryBroken) << '\n';
      }
      catch(const std::exception&)
      {
      }
    }

    const TiePie::Hardware::Oscilloscope* scp = dynamic_cast<const TiePie::Hardware::Oscilloscope*>(&dev);
    if(scp)
      printOscilloscopeInfo(*scp);

    const TiePie::Hardware::Generator* gen = dynamic_cast<const TiePie::Hardware::Generator*>(&dev);
    if(gen)
      printGeneratorInfo(*gen);
  }
  catch(const std::exception& e)
  {
    std::cerr << "Exception: " << e.what() << '\n';
  }
}

void printOscilloscopeInfo(const TiePie::Hardware::Oscilloscope& scp)
{
  try
  {
    std::cout << "Oscilloscope:\n";
    std::cout << "  Channel count             : " << scp.channels.count() << '\n';
    std::cout << "  SureConnect               : " << boolToStr(scp.hasSureconnect()) << '\n';
    std::cout << "  Measure modes             : " << measureModeToStr(scp.measureModes()) << '\n';
    std::cout << "  Measure mode              : " << measureModeToStr(scp.measureMode()) << '\n';
    std::cout << "  Auto resolution modes     : " << autoResolutionModeToStr(scp.autoResolutionModes()) << '\n';
    std::cout << "  Auto resolution mode      : " << autoResolutionModeToStr(scp.autoResolutionMode()) << '\n';

    std::cout << "  Resolutions               : ";
    const std::vector<uint8_t> Resolutions(scp.resolutions());
    for(std::vector<uint8_t>::size_type i = 0; i < Resolutions.size(); i++)
    {
      if(i != 0)
        std::cout << ", ";

      std::cout << (unsigned int)Resolutions[i];
    }
    std::cout << '\n';

    std::cout << "  Resolution                : " << (unsigned int)scp.resolution() << '\n';
    std::cout << "  Resolution enhanced       : " << boolToStr(scp.isResolutionEnhanced()) << '\n';
    std::cout << "  Clock outputs             : " << clockOutputToStr(scp.clockOutputs()) << '\n';
    std::cout << "  Clock output              : " << clockOutputToStr(scp.clockOutput()) << '\n';

    try
    {
      const std::vector<double> ClockOutputFrequencies(scp.clockOutputFrequencies());
      std::cout << "  Clock output frequencies  : ";
      for(std::vector<double>::size_type i = 0; i < ClockOutputFrequencies.size(); i++)
      {
        if(i != 0)
          std::cout << ", ";

        std::cout << ClockOutputFrequencies[i];
      }
      std::cout << '\n';
      std::cout << "  Clock output frequency    : " << scp.clockOutputFrequency() << '\n';
    }
    catch(const std::exception&)
    {
    }

    std::cout << "  Clock sources             : " << clockSourceToStr(scp.clockSources()) << '\n';
    std::cout << "  Clock source              : " << clockSourceToStr(scp.clockSource()) << '\n';

    try
    {
      const std::vector<double> ClockSourceFrequencies(scp.clockSourceFrequencies());
      std::cout << "  Clock source frequencies  : ";
      for(std::vector<double>::size_type i = 0; i < ClockSourceFrequencies.size(); i++)
      {
        if(i != 0)
          std::cout << ", ";

        std::cout << ClockSourceFrequencies[i];
      }
      std::cout << '\n';
      std::cout << "  Clock source frequency      : " << scp.clockSourceFrequency() << '\n';
    }
    catch(const std::exception&)
    {
    }

    std::cout << "  Record length max         : " << scp.recordLengthMax() << '\n';
    std::cout << "  Record length             : " << scp.recordLength() << '\n';
    std::cout << "  Sample rate max           : " << scp.sampleRateMax() << '\n';
    std::cout << "  Sample rate               : " << scp.sampleRate() << '\n';

    if(scp.measureMode() == TIEPIE_HW_MM_BLOCK)
    {
      std::cout << "  Segment count max         : " << scp.segmentCountMax() << '\n';
      if(scp.segmentCountMax() > 1)
        std::cout << "  Segment count             : " << scp.segmentCount() << '\n';
    }

    if(scp.hasTrigger())
    {
      std::cout << "  Pre sample ratio          : " << scp.preSampleRatio() << '\n';

      std::cout << "  Trigger time out          : ";
      if(scp.trigger->timeout() == TIEPIE_HW_TO_INFINITY)
        std::cout << "Infinite";
      else
        std::cout << scp.trigger->timeout();
      std::cout << '\n';

      if(scp.trigger->hasDelay())
      {
        std::cout << "  Trigger delay max         : " << scp.trigger->delayMax() << '\n';
        std::cout << "  Trigger delay             : " << scp.trigger->delay() << '\n';
      }

      if(scp.hasPresamplesValid())
        std::cout << "  Presamples valid          : " << boolToStr(scp.presamplesValid()) << '\n';
    }

    const uint16_t channelCount = scp.channels.count();
    for(uint16_t ch = 0; ch < channelCount; ch++)
    {
      const std::unique_ptr<TiePie::Hardware::OscilloscopeChannel>& channel = scp.channels[ch];
      std::cout << "  Ch" << (ch + 1) << ":\n";
      std::cout << "    Connector type          : " << connectorTypeToStr(channel->connectorType()) << '\n';
      std::cout << "    Differential            : " << boolToStr(channel->isDifferential()) << '\n';
      std::cout << "    Impedance               : " << channel->impedance() << '\n';
      std::cout << "    Connection test         : " << boolToStr(channel->hasSureconnect()) << '\n';
      std::cout << "    Available               : " << boolToStr(channel->isAvailable()) << '\n';
      std::cout << "    Enabled                 : " << boolToStr(channel->enabled()) << '\n';

      std::cout << "    Bandwidths              : ";
      const std::vector<double> Bandwidths(channel->bandwidths());
      for(std::vector<double>::size_type i = 0; i < Bandwidths.size(); i++)
      {
        if(i != 0)
          std::cout << ", ";

        std::cout << Bandwidths[i];
      }
      std::cout << '\n';
      std::cout << "    Bandwidth               : " << channel->bandwidth() << '\n';

      std::cout << "    Couplings               : " << couplingToStr(channel->couplings()) << '\n';
      std::cout << "    Coupling                : " << couplingToStr(channel->coupling()) << '\n';
      std::cout << "    Auto ranging            : " << boolToStr(channel->autoRanging()) << '\n';

      std::cout << "    Ranges                  : ";
      const std::vector<double> Ranges(channel->ranges());
      for(std::vector<double>::size_type i = 0; i < Ranges.size(); i++)
      {
        if(i != 0)
          std::cout << ", ";

        std::cout << Ranges[i];
      }
      std::cout << '\n';

      std::cout << "    Range                   : " << channel->range() << '\n';
      if(channel->hasSafeground())
      {
        std::cout << "    SafeGround enabled      : " << boolToStr(channel->safegroundEnabled()) << '\n';
        std::cout << "    SafeGround threshold max: " << channel->safegroundThresholdMax() << '\n';
        std::cout << "    SafeGround threshold min: " << channel->safegroundThresholdMin() << '\n';
        std::cout << "    SafeGround threshold    : " << channel->safegroundThreshold() << '\n';
      }

      if(channel->hasTrigger())
      {
        std::cout << "    Trigger:\n";
        std::cout << "      Available             : " << boolToStr(channel->trigger->isAvailable()) << '\n';
        std::cout << "      Enabled               : " << boolToStr(channel->trigger->enabled()) << '\n';
        std::cout << "      Kinds                 : " << triggerKindToStr(channel->trigger->kinds()) << '\n';
        std::cout << "      Kind                  : " << triggerKindToStr(channel->trigger->kind()) << '\n';
        std::cout << "      Level modes           : " << triggerLevelModeToStr(channel->trigger->levelModes()) << '\n';
        std::cout << "      Level mode            : " << triggerLevelModeToStr(channel->trigger->levelMode()) << '\n';

        const uint32_t triggerLevelCount = channel->trigger->levels.count();
        std::cout << "      Levels                : ";
        for(uint32_t i = 0; i < triggerLevelCount; i++)
        {
          if(i != 0)
            std::cout << ", ";

          std::cout << *channel->trigger->levels[i];
        }
        std::cout << '\n';

        const uint32_t triggerHysteresisCount = channel->trigger->hystereses.count();
        std::cout << "      Hystereses            : ";
        for(uint32_t i = 0; i < triggerHysteresisCount; i++)
        {
          if(i != 0)
            std::cout << ", ";

          std::cout << *channel->trigger->hystereses[i];
        }
        std::cout << '\n';

        uint32_t triggerConditions = channel->trigger->conditions();
        std::cout << "      Conditions            : " << triggerConditionToStr(triggerConditions) << '\n';
        if(triggerConditions != TIEPIE_HW_TCM_NONE)
          std::cout << "      Condition             : " << triggerConditionToStr(channel->trigger->condition()) << '\n';

        const uint32_t triggerTimeCount = channel->trigger->times.count();
        if(triggerTimeCount > 0)
        {
          std::cout << "      Times                 : ";
          for(uint32_t i = 0; i < triggerTimeCount; i++)
          {
            if(i != 0)
              std::cout << ", ";

            std::cout << *channel->trigger->times[i];
          }
          std::cout << '\n';
        }
      }
    }

    printTriggerInputsInfo(scp.triggerInputs);
    printTriggerOutputsInfo(scp.triggerOutputs);
  }
  catch(const std::exception& e)
  {
    std::cerr << "Exception: " << e.what() << '\n';
  }
}

void printGeneratorInfo(const TiePie::Hardware::Generator& gen)
{
  try
  {
    std::cout << "Generator:\n";
    std::cout << "  Connector type            : " << connectorTypeToStr(gen.connectorType()) << '\n';
    std::cout << "  Differential              : " << boolToStr(gen.isDifferential()) << '\n';
    std::cout << "  Controllable              : " << boolToStr(gen.isControllable()) << '\n';
    std::cout << "  Impedance                 : " << gen.impedance() << '\n';
    std::cout << "  Resolution                : " << (unsigned int)gen.resolution() << '\n';
    std::cout << "  Output value min          : " << gen.outputValueMin() << '\n';
    std::cout << "  Output value max          : " << gen.outputValueMax() << '\n';
    std::cout << "  Output on                 : " << boolToStr(gen.outputEnable()) << '\n';
    if(gen.hasOutputInvert())
    {
      std::cout << "  Output invert             : " << boolToStr(gen.outputInvert()) << '\n';
    }

    std::cout << "  Modes native              : " << generatorModeToStr(gen.modesNative()) << '\n';
    std::cout << "  Modes                     : " << generatorModeToStr(gen.modes()) << '\n';
    if(gen.modes() != TIEPIE_HW_GMM_NONE)
    {
      std::cout << "  Mode                      : " << generatorModeToStr(gen.mode()) << '\n';
      if(gen.mode() & TIEPIE_HW_GMM_BURST_COUNT)
      {
        std::cout << "  Burst active              : " << boolToStr(gen.isBurstActive()) << '\n';
        std::cout << "  Burst count max           : " << gen.burstCountMax() << '\n';
        std::cout << "  Burst count               : " << gen.burstCount() << '\n';
      }
      if(gen.mode() & TIEPIE_HW_GMM_BURST_SAMPLE_COUNT)
      {
        std::cout << "  Burst sample count max    : " << gen.burstSampleCountMax() << '\n';
        std::cout << "  Burst sample count        : " << gen.burstSampleCount() << '\n';
      }
      if(gen.mode() & TIEPIE_HW_GMM_BURST_SEGMENT_COUNT)
      {
        std::cout << "  Burst segment count max   : " << gen.burstSegmentCountMax() << '\n';
        std::cout << "  Burst segment count       : " << gen.burstSegmentCount() << '\n';
      }
    }

    std::cout << "  Signal types              : " << signalTypeToStr(gen.signalTypes()) << '\n';
    std::cout << "  Signal type               : " << signalTypeToStr(gen.signalType()) << '\n';

    if(gen.hasAmplitude())
    {
      std::cout << "  Amplitude min             : " << gen.amplitudeMin() << '\n';
      std::cout << "  Amplitude max             : " << gen.amplitudeMax() << '\n';
      std::cout << "  Amplitude                 : " << gen.amplitude() << '\n';

      std::cout << "  Amplitude ranges          : ";
      const std::vector<double> AmplitudeRanges(gen.amplitudeRanges());
      for(std::vector<double>::size_type i = 0; i < AmplitudeRanges.size(); i++)
      {
        if(i != 0)
          std::cout << ", ";

        std::cout << AmplitudeRanges[i];
      }
      std::cout << '\n';

      std::cout << "  Amplitude range           : " << gen.amplitudeRange() << '\n';
      std::cout << "  Amplitude auto ranging    : " << boolToStr(gen.amplitudeAutoRanging()) << '\n';
    }

    if(gen.hasFrequency())
    {
      std::cout << "  Frequency modes           : " << frequencyModeToStr(gen.frequencyModes()) << '\n';
      std::cout << "  Frequency mode            : " << frequencyModeToStr(gen.frequencyMode()) << '\n';
      std::cout << "  Frequency min             : " << gen.frequencyMin() << '\n';
      std::cout << "  Frequency max             : " << gen.frequencyMax() << '\n';
      std::cout << "  Frequency                 : " << gen.frequency() << '\n';
    }

    if(gen.hasOffset())
    {
      std::cout << "  Offset min                : " << gen.offsetMin() << '\n';
      std::cout << "  Offset max                : " << gen.offsetMax() << '\n';
      std::cout << "  Offset                    : " << gen.offset() << '\n';
    }

    if(gen.hasPhase())
    {
      std::cout << "  Phase min                 : " << gen.phaseMin() << '\n';
      std::cout << "  Phase max                 : " << gen.phaseMax() << '\n';
      std::cout << "  Phase                     : " << gen.phase() << '\n';
    }

    if(gen.hasSymmetry())
    {
      std::cout << "  Symmetry min              : " << gen.symmetryMin() << '\n';
      std::cout << "  Symmetry max              : " << gen.symmetryMax() << '\n';
      std::cout << "  Symmetry                  : " << gen.symmetry() << '\n';
    }

    if(gen.hasWidth())
    {
      std::cout << "  Width min                 : " << gen.widthMin() << '\n';
      std::cout << "  Width max                 : " << gen.widthMax() << '\n';
      std::cout << "  Width                     : " << gen.width() << '\n';
    }

    if(gen.hasEdgeTime())
    {
      std::cout << "  Leading edge time min     : " << gen.leadingEdgeTimeMin() << '\n';
      std::cout << "  Leading edge time max     : " << gen.leadingEdgeTimeMax() << '\n';
      std::cout << "  Leading edge time         : " << gen.leadingEdgeTime() << '\n';
      std::cout << "  Trailing edge time min    : " << gen.trailingEdgeTimeMin() << '\n';
      std::cout << "  Trailing edge time max    : " << gen.trailingEdgeTimeMax() << '\n';
      std::cout << "  Trailing edge time        : " << gen.trailingEdgeTime() << '\n';
    }

    if(gen.hasData())
    {
      std::cout << "  DataLength min            : " << gen.dataLengthMin() << '\n';
      std::cout << "  DataLength max            : " << gen.dataLengthMax() << '\n';
      std::cout << "  DataLength                : " << gen.dataLength() << '\n';
    }

    printTriggerInputsInfo(gen.triggerInputs);
    printTriggerOutputsInfo(gen.triggerOutputs);
  }
  catch(const std::exception& e)
  {
    std::cerr << "Exception: " << e.what() << '\n';
  }
}

void printServerInfo(const TiePie::Hardware::Server& srv)
{
  try
  {
    std::cout << "Server:\n";
    std::cout << "  URL                       : " << srv.url() << '\n';
    std::cout << "  Name                      : " << srv.name() << '\n';
    std::cout << "  Description               : " << srv.description() << '\n';
    std::cout << "  IP address                : " << srv.ipAddress() << '\n';
    std::cout << "  IP port                   : " << srv.ipPort() << '\n';
    std::cout << "  Id                        : " << srv.id() << '\n';
    std::cout << "  Version                   : " << srv.version() << '\n';
    std::cout << "  Status                    : " << ServerStatuses[srv.status()] << '\n';
    if(srv.lastError() != TIEPIE_HW_SERVER_ERROR_NONE)
      std::cout << "  Last error                : " << ServerErrorCodes[srv.lastError()] << '\n';
  }
  catch(const std::exception& e)
  {
    std::cerr << "Exception: " << e.what() << '\n';
  }
}

void printTriggerInputsInfo(const TiePie::Hardware::TriggerInputs& triggerInputs)
{
  const TiePie::Hardware::TriggerInputs::size_type Count = triggerInputs.count();

  for(TiePie::Hardware::TriggerInputs::size_type i = 0; i < Count; i++)
  {
    const std::unique_ptr<TiePie::Hardware::TriggerInput>& triggerInput = triggerInputs[i];

    std::cout << "  TriggerInput " << i << ":" << '\n';
    std::cout << "    Id                      : " << triggerInput->id() << '\n';
    std::cout << "    Name                    : " << triggerInput->name() << '\n';
    std::cout << "    Available               : " << boolToStr(triggerInput->isAvailable()) << '\n';

    if(triggerInput->isAvailable())
    {
      std::cout << "    Enabled                 : " << boolToStr(triggerInput->enabled()) << '\n';
      std::cout << "    Kinds                   : " << triggerKindToStr(triggerInput->kinds()) << '\n';
      if(triggerInput->kinds() != TIEPIE_HW_TKM_NONE)
        std::cout << "    Kind                    : " << triggerKindToStr(triggerInput->kind()) << '\n';
    }
  }
}

void printTriggerOutputsInfo(const TiePie::Hardware::TriggerOutputs& triggerOutputs)
{
  const TiePie::Hardware::TriggerOutputs::size_type Count = triggerOutputs.count();

  for(TiePie::Hardware::TriggerOutputs::size_type i = 0; i < Count; i++)
  {
    const std::unique_ptr<TiePie::Hardware::TriggerOutput>& triggerOutput = triggerOutputs[i];

    std::cout << "  TriggerOutput " << i << ":\n";
    std::cout << "    Id                      : " << triggerOutput->id() << '\n';
    std::cout << "    Name                    : " << triggerOutput->name() << '\n';
    std::cout << "    Enabled                 : " << boolToStr(triggerOutput->enabled()) << '\n';
    std::cout << "    Events                  : " << triggerOutputEventToStr(triggerOutput->events()) << '\n';
    std::cout << "    Event                   : " << triggerOutputEventToStr(triggerOutput->event()) << '\n';
  }
}

// String conversion functions:
std::string generatorModeToStr(const uint64_t generatorModes)
{
  if(generatorModes == TIEPIE_HW_GM_UNKNOWN)
    return "Unknown";

  std::string s;

  for(unsigned int i = 0; i < GeneratorModes.size(); i++)
  {
    if(generatorModes & (1ULL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(GeneratorModes[i]);
    }
  }

  return s;
}

std::string clockOutputToStr(const uint32_t clockOutputs)
{
  std::string s;

  for(unsigned int i = 0; i < ClockOutputTypes.size(); i++)
  {
    if(clockOutputs & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(ClockOutputTypes[i]);
    }
  }

  return s;
}

std::string clockSourceToStr(const uint32_t clockSources)
{
  std::string s;

  for(unsigned int i = 0; i < ClockSources.size(); i++)
  {
    if(clockSources & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(ClockSources[i]);
    }
  }

  return s;
}

std::string connectorTypeToStr(const uint32_t connectorTypes)
{
  if(connectorTypes == TIEPIE_HW_CONNECTORTYPE_UNKNOWN)
    return "Unknown";

  std::string s;

  for(unsigned int i = 0; i < ConnectorTypes.size(); i++)
  {
    if(connectorTypes & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(ConnectorTypes[i]);
    }
  }

  return s;
}

std::string dateToStr(const tiepie_hw_date& date)
{
  if(date.year == 0 && date.month == 0 && date.day == 0)
    return "Unavailable";

  return std::to_string(date.year) + '-' + std::to_string(date.month) + '-' + std::to_string(date.day);
}

std::string versionToStr(const tiepie_hw_version& version)
{
  return std::to_string(version.major) + '.' + std::to_string(version.minor) + '.' + std::to_string(version.patch) + '.' + std::to_string(version.build) + version.extra;
}

std::string couplingToStr(const uint64_t couplings)
{
  if(couplings == TIEPIE_HW_CK_UNKNOWN)
    return "Unknown";

  std::string s;

  for(unsigned int i = 0; i < Couplings.size(); i++)
  {
    if(couplings & (1ULL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(Couplings[i]);
    }
  }

  return s;
}

std::string deviceTypeToStr(const uint32_t deviceTypes)
{
  std::string s;

  for(unsigned int i = 0; i < DeviceTypes.size(); i++)
  {
    if(deviceTypes & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(DeviceTypes[i]);
    }
  }

  return s;
}

std::string frequencyModeToStr(const uint32_t frequencyModes)
{
  if(frequencyModes == TIEPIE_HW_FM_UNKNOWN)
    return "Unknown";

  std::string s;

  for(unsigned int i = 0; i < FrequencyModes.size(); i++)
  {
    if(frequencyModes & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(FrequencyModes[i]);
    }
  }

  return s;
}

std::string measureModeToStr(const uint32_t measureModes)
{
  if(measureModes == TIEPIE_HW_MM_UNKNOWN)
    return "Unknown";

  std::string s;

  for(unsigned int i = 0; i < MeasureModes.size(); i++)
  {
    if(measureModes & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(MeasureModes[i]);
    }
  }

  return s;
}

std::string autoResolutionModeToStr(const uint32_t autoResolutions)
{
  if(autoResolutions == TIEPIE_HW_ARM_UNKNOWN)
    return "Unknown";

  std::string s;

  for(unsigned int i = 0; i < AutoResolutionModes.size(); i++)
  {
    if(autoResolutions & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(AutoResolutionModes[i]);
    }
  }

  return s;
}

std::string signalTypeToStr(const uint32_t signalTypes)
{
  if(signalTypes == TIEPIE_HW_ST_UNKNOWN)
    return "Unknown";

  std::string s;

  for(unsigned int i = 0; i < SignalTypes.size(); i++)
  {
    if(signalTypes & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(SignalTypes[i]);
    }
  }

  return s;
}

std::string triggerConditionToStr(const uint32_t triggerConditions)
{
  if(triggerConditions == TIEPIE_HW_TC_UNKNOWN)
    return "Unknown";

  std::string s;

  for(unsigned int i = 0; i < TriggerConditions.size(); i++)
  {
    if(triggerConditions & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(TriggerConditions[i]);
    }
  }

  return s;
}

std::string triggerKindToStr(const uint64_t triggerKinds)
{
  if(triggerKinds == TIEPIE_HW_TK_UNKNOWN)
    return "Unknown";

  std::string s;

  for(size_t i = 0; i < TriggerKinds.size(); i++)
  {
    if(triggerKinds & (1ULL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(TriggerKinds[i]);
    }
  }

  return s;
}

std::string triggerLevelModeToStr(const uint32_t triggerLevelModes)
{
  if(triggerLevelModes == TIEPIE_HW_TLM_UNKNOWN)
    return "Unknown";

  std::string s;

  for(size_t i = 0; i < TriggerLevelModes.size(); i++)
  {
    if(triggerLevelModes & (1ULL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(TriggerLevelModes[i]);
    }
  }

  return s;
}

std::string triggerOutputEventToStr(const uint64_t triggerEvents)
{
  if(triggerEvents == TIEPIE_HW_TOE_UNKNOWN)
    return "Unknown";

  std::string s;

  for(size_t i = 0; i < TriggerOutputEvents.size(); i++)
  {
    if(triggerEvents & (1ULL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(TriggerOutputEvents[i]);
    }
  }

  return s;
}

std::string boolToStr(const bool value)
{
  return value ? "true" : "false";
}
